-- Project: Fractal art
-- SDK: Corona - 09/12/11
-- Author: Andrew Burch
-- Date: 11/01/2012
-- Site: http://www.newhighscore.net
-- Contact: andrew.burch@newhighscore.net
-- Note: Simple Mandelbrot and Julia sets

local renderer = require('renderer')
local mandelbrot = require('mandelbrot')
local julia = require('julia')


-- hide the status bar
display.setStatusBar(display.HiddenStatusBar)

local contentWidth = display.contentWidth
local contentHeight = display.contentHeight

local cellWidth = 2
local cellHeight = 2
local outputWidth = math.floor(contentWidth / cellWidth)
local outputHeight = math.floor(contentHeight / cellHeight)
local maxIterations = 128

-- initialise renderer
local renderSystem = renderer.new()
renderSystem:initialise {
			outputWidth = outputWidth,
			outputHeight = outputHeight,
			cellWidth = cellWidth,
			cellHeight = cellHeight,
		}

-- initialise sets		
local mandelbrotSet = mandelbrot.new()
mandelbrotSet:calculateSet {
			maxIterations = maxIterations,
			realAxisMinimum = -2,
			realAxisMaximum = 1,
			imaginaryAxisMinimum = -1,
			imaginaryAxisMaximum = 1,
			outputWidth = outputWidth,
			outputHeight = outputHeight,
		}

local juliaSet = julia.new()
juliaSet:calculateSet {
			maxIterations = maxIterations,
			realAxisMinimum = -1.5,
			realAxisMaximum = 1.5,
			imaginaryAxisMinimum = -1.5,
			imaginaryAxisMaximum = 1.5,
			seedRealValue = -0.4,
			seedImaginaryValue = 0.6,
			outputWidth = outputWidth,
			outputHeight = outputHeight,
		}

local sets = {
	{
		set = mandelbrotSet:getSet(),
		name = 'Mandelbrot',
	},
	{
		set = juliaSet:getSet(),
		name = 'Julia',
	},
}

local maxSets = #sets
local currentSetId = 1

renderSystem:setRenderSet(sets[currentSetId].set)


-- set up a toggle button to switch between sets
local btnWidth = 50
local btnHeight = 20
local sideBorder = 10
local btnX = contentWidth - btnWidth - sideBorder
local btnY = contentHeight - btnHeight - sideBorder
local button = display.newRoundedRect(btnX, btnY, btnWidth, btnHeight, 3)
button:setFillColor(140, 140, 140, 100)
button:setStrokeColor(180, 180, 180, 100)
local btnText = display.newText('Toggle', btnX + 5, btnY + 3, native.systemFont, 12)
local setText = display.newText('Mandelbrot', sideBorder, btnY + 3, native.systemFont, 12)
button:addEventListener('touch', function(event)
							local phase = event.phase
							if phase ~= 'began' then
								return 
							end
							
							local nextSetId = currentSetId + 1
							if nextSetId > maxSets then
								nextSetId = 1
							end
							
							local nextSetInfo = sets[nextSetId]
							renderSystem:setRenderSet(nextSetInfo.set)
							setText.text = nextSetInfo.name
							
							currentSetId = nextSetId
									
							return true
						end)


local lastUpdateTime = 0

-- register main update listener
Runtime:addEventListener("enterFrame", function(event) 
											local time = event.time
											local dt = (time - lastUpdateTime) / 1000
											
											lastUpdateTime = time											
										end)
